# Generated from Dynabuffers.g4 by ANTLR 4.7
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .DynabuffersParser import DynabuffersParser
else:
    from DynabuffersParser import DynabuffersParser

# This class defines a complete listener for a parse tree produced by DynabuffersParser.
class DynabuffersListener(ParseTreeListener):

    # Enter a parse tree produced by DynabuffersParser#compilation.
    def enterCompilation(self, ctx:DynabuffersParser.CompilationContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#compilation.
    def exitCompilation(self, ctx:DynabuffersParser.CompilationContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#enumType.
    def enterEnumType(self, ctx:DynabuffersParser.EnumTypeContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#enumType.
    def exitEnumType(self, ctx:DynabuffersParser.EnumTypeContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#classType.
    def enterClassType(self, ctx:DynabuffersParser.ClassTypeContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#classType.
    def exitClassType(self, ctx:DynabuffersParser.ClassTypeContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#unionType.
    def enterUnionType(self, ctx:DynabuffersParser.UnionTypeContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#unionType.
    def exitUnionType(self, ctx:DynabuffersParser.UnionTypeContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#fieldType.
    def enterFieldType(self, ctx:DynabuffersParser.FieldTypeContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#fieldType.
    def exitFieldType(self, ctx:DynabuffersParser.FieldTypeContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#dataType.
    def enterDataType(self, ctx:DynabuffersParser.DataTypeContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#dataType.
    def exitDataType(self, ctx:DynabuffersParser.DataTypeContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#arrayType.
    def enterArrayType(self, ctx:DynabuffersParser.ArrayTypeContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#arrayType.
    def exitArrayType(self, ctx:DynabuffersParser.ArrayTypeContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#optionType.
    def enterOptionType(self, ctx:DynabuffersParser.OptionTypeContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#optionType.
    def exitOptionType(self, ctx:DynabuffersParser.OptionTypeContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#namespaceType.
    def enterNamespaceType(self, ctx:DynabuffersParser.NamespaceTypeContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#namespaceType.
    def exitNamespaceType(self, ctx:DynabuffersParser.NamespaceTypeContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#classOptions.
    def enterClassOptions(self, ctx:DynabuffersParser.ClassOptionsContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#classOptions.
    def exitClassOptions(self, ctx:DynabuffersParser.ClassOptionsContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#unionOptions.
    def enterUnionOptions(self, ctx:DynabuffersParser.UnionOptionsContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#unionOptions.
    def exitUnionOptions(self, ctx:DynabuffersParser.UnionOptionsContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#fieldOptions.
    def enterFieldOptions(self, ctx:DynabuffersParser.FieldOptionsContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#fieldOptions.
    def exitFieldOptions(self, ctx:DynabuffersParser.FieldOptionsContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#annotation.
    def enterAnnotation(self, ctx:DynabuffersParser.AnnotationContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#annotation.
    def exitAnnotation(self, ctx:DynabuffersParser.AnnotationContext):
        pass


    # Enter a parse tree produced by DynabuffersParser#value.
    def enterValue(self, ctx:DynabuffersParser.ValueContext):
        pass

    # Exit a parse tree produced by DynabuffersParser#value.
    def exitValue(self, ctx:DynabuffersParser.ValueContext):
        pass


