# This file is autogenerated from DynabuffersVersion.vm

class VersionWrapper:
    def __init__(self, version: str):
        self._version = version

    def get_major(self) -> int:
        try:
            return int(self._version.split(".")[0])
        except ValueError as err:
            raise ValueError("Dynabuffers major version cannot be extracted from: {}".format(self._version)) from err


DYNABUFFERS_VERSION = VersionWrapper("2.6.1")
